<?php

namespace App\domain\amadeus;

use App\Traits\FileStorageTrait;
use Carbon\Carbon;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;

class AddPNRMultiElementAction extends AmadeusRequestInSeries
{
    use FileStorageTrait;

    protected Collection $travellers;
    protected string $leadTelephoneNumber;
    protected string $leadEmail;

    protected int $sequenceNumber;
    protected string $amadeusSessionId;
    protected string $amadeusSecurityToken;

    public function __construct()
    {
        parent::__construct();

        $this->sequenceNumber = 2;
    }

    public function execute(array $travellers, string $amadeusSessionId, string $amadeusSecurityToken): array
    {
        try {
            $this->amadeusSessionId = $amadeusSessionId;
            $this->amadeusSecurityToken = $amadeusSecurityToken;
            $this->travellers = Collection::wrap($travellers);

            $this->leadTelephoneNumber = $travellers[0]['Traveler']['Telephone'][0]['phoneNumber'];
            $this->leadEmail = $travellers[0]['Traveler']['Email'][0]['value'];

            $response = $this->createCurlRequest();

            self::storeInFile("xml", "2.2-add-pnr-multi-element-response", $response);

            $responseArray = $this->xmlResponseBodyToArray($response);

            self::storeInFile(
                "json", "2.2-add-pnr-multi-element-response", json_encode($responseArray)
            );

            return $responseArray;

        } catch (\Exception $exception) {
            $e = "Error in AddPNRMultiElementsAction execute:" . $exception->getMessage();
            Log::error($e);
            echo $e;

            return ['error' => $e];
        }
    }

    protected function action(): string
    {
        return "http://webservices.amadeus.com/PNRADD_21_1_1A";
    }

    protected function generateXMLRequest(): string
    {
        $xmlRequest = $this->xmlHeader($this->sequenceNumber, $this->amadeusSessionId, $this->amadeusSecurityToken) .
            '
              <soap:Body>
              <PNR_AddMultiElements>
                <pnrActions>
                  <optionCode>0</optionCode>
                </pnrActions>
            ' . $this->generateTravellerInfoTag() . '

                 <dataElementsMaster>
                      <marker1/>

                    ' . $this->getDataElementMasterTags() . '
                    ' . $this->generateEmailDataElementTags() . '

                    <dataElementsIndiv>
                      <elementManagementData>
                        <segmentName>RF</segmentName>
                      </elementManagementData>
                      <freetextData>
                        <freetextDetail>
                          <subjectQualifier>3</subjectQualifier>
                          <type>P23</type>
                        </freetextDetail>
                        <longFreetext>ThisisExample</longFreetext>
                      </freetextData>
                    </dataElementsIndiv>

                 </dataElementsMaster>
              </PNR_AddMultiElements>
            </soap:Body>
            </soap:Envelope>
        ';


        self::storeInFile("xml", "2.2-add-pnr-multi-element-request", $xmlRequest);

        return $xmlRequest;
    }

    protected function generateTravellerInfoTag(): string
    {
        $adultsAndChildren = $this->travellers->whereIn("Traveler.passengerTypeCode", ["ADT", "CHD"])->values();
        $infants = $this->travellers->where("Traveler.passengerTypeCode", "INF")->values();

        $xml = "";
        foreach ($adultsAndChildren as $index => $traveller) {
            $birthdayObj = Carbon::parse($traveller['Traveler']['birthDate']);
            $birthday = $birthdayObj->format("d") . $birthdayObj->format("M") . $birthdayObj->format("y");

            $xml .= '
            <travellerInfo>
                  <elementManagementPassenger>
                    <reference>
                      <qualifier>PR</qualifier>
                      <number>' . ($index + 1) . '</number>
                    </reference>
                    <segmentName>NM</segmentName>
                  </elementManagementPassenger>
                  <passengerData>
                    <travellerInformation>
                          <traveller>
                            <surname>' . $traveller['Traveler']['PersonName']['Surname'] . '</surname>
                            <quantity>1</quantity>
                          </traveller>
                          <passenger>
                            <firstName>' . $traveller['Traveler']['PersonName']['Given'] . '</firstName>
                            <type>' . $traveller['Traveler']['passengerTypeCode'] . '</type>
                            ' . (isset($infants[$index]) ? '<infantIndicator>3</infantIndicator>' : '') . '
                          </passenger>

                      </travellerInformation>

                      '
                .
                (isset($infants[$index])
                    ? ""
                    : '
                                  <dateOfBirth>
                                    <dateAndTimeDetails>
                                        <date>' . $birthday . '</date>
                                    </dateAndTimeDetails>
                                  </dateOfBirth>
                                  ')
                .
                '
                  </passengerData>
                   ' . (isset($infants[$index]) ? $this->getInfantTag($infants[$index]) : "") . '
            </travellerInfo>
            ';
        }

        return $xml;
    }

    protected function getInfantTag($infant): string
    {
        $birthdayObj = Carbon::parse($infant['Traveler']['birthDate']);
        $birthday = $birthdayObj->format("d") . $birthdayObj->format("M") . $birthdayObj->format("y");

        return "
            <passengerData>
              <travellerInformation>
                  <traveller>
                      <surname>{$infant['Traveler']['PersonName']['Surname']}</surname>
                  </traveller>
                  <passenger>
                      <firstName>{$infant['Traveler']['PersonName']['Given']}</firstName>
                      <type>INF</type>
                  </passenger>
              </travellerInformation>
              <dateOfBirth>
                  <dateAndTimeDetails>
                      <date>$birthday</date>
                  </dateAndTimeDetails>
              </dateOfBirth>
            </passengerData>
        ";
    }

    protected function getDataElementMasterTags(): string
    {
        $allowedPassengerTypes = ["ADT", "CHD"];
        $adultsAndChildren = $this->travellers->whereIn("Traveler.passengerTypeCode", $allowedPassengerTypes)->values();
        $infants = $this->travellers->where("Traveler.passengerTypeCode", "INF")->values();

        $xml = "";
        foreach ($adultsAndChildren as $index => $traveller) {
            $xml = $this->generateDataElementsIndiv($traveller['Traveler'], $index, $xml);
        }

        foreach ($infants as $index => $traveller) {
            $xml = $this->generateDataElementsIndiv($traveller['Traveler'], $index, $xml);
        }

        return $xml;
    }

    protected function generateDataElementsIndiv($traveler, int|string|null $index, string $xml): string
    {
        $birthdayObj = Carbon::parse($traveler['birthDate']);
        $birthday = $birthdayObj->format("d") . $birthdayObj->format("M") . $birthdayObj->format("y");

        $givenNames = "";
        foreach (explode(' ', $traveler['PersonName']['Given']) as $name) {
            $givenNames .= "-";
            $givenNames .= $name;
        }

        $ppExpireDateCarbon = Carbon::parse($traveler['TravelDocument'][0]['expireDate']);
        $ppExpireDate = $ppExpireDateCarbon->format("d") .
            $ppExpireDateCarbon->format("M") .
            $ppExpireDateCarbon->format("y");


        $passportText = "P" .
            "-" . $traveler['TravelDocument'][0]['issueCountry'] .
            "-" . $traveler['TravelDocument'][0]['docNumber'] .
            "-" . $traveler['TravelDocument'][0]['nationality'] .
            "-" . $birthday .
            "-" . ($traveler["gender"] === "Male" ? "M" : "F") .
            "-" . $ppExpireDate .
            "-" . $traveler['PersonName']['Surname'] .
            "-" . $traveler['PersonName']['Given'] .
            "-" . $traveler['PersonName']['Middle'];

        $xml .= '
                <dataElementsIndiv>
                  <elementManagementData>
                    <segmentName>SSR</segmentName>
                  </elementManagementData>
                  <serviceRequest>
                    <ssr>
                      <type>DOCS</type>
                      <status>HK</status>
                      <quantity>1</quantity>
                      <companyId>YY</companyId>
                      <freetext>' . $passportText . '</freetext>
                    </ssr>
                  </serviceRequest>
                  <referenceForDataElement>
                    <reference>
                      <qualifier>PR</qualifier>
                      <number>' . ($index + 1) . '</number>
                    </reference>
                  </referenceForDataElement>
                </dataElementsIndiv>
            ';
        return $xml;
    }

    protected function generateEmailDataElementTags(): string
    {
        $email = $this->travellers[0]['Traveler']['Email'][0]['value'];

        return '
            <dataElementsIndiv>
             <elementManagementData>
                <reference>
                   <qualifier>PR</qualifier>
                   <number>1</number>
                </reference>
                <segmentName>AP</segmentName>
             </elementManagementData>
             <freetextData>
                <freetextDetail>
                   <subjectQualifier>3</subjectQualifier>
                   <type>P02</type>
                </freetextDetail>
                <longFreetext>' . $email . '</longFreetext>
              </freetextData>
            </dataElementsIndiv>
        ';
    }
}

