<?php

namespace App\domain\amadeus;

use App\domain\amadeus\traits\AmadeusParamsTraits;
use App\Traits\FileStorageTrait;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;

class SelFromRecommendationAction extends AmadeusRequestStateless
{
    use FileStorageTrait;

    protected int $sequenceNumber;
    protected array $selectedSegments;
    protected array $passengerCriteria;
    protected array $searchCriteria;
    protected array $recommendation;

    public function __construct()
    {
        parent::__construct();

        $this->sequenceNumber = 4;
    }

    public function execute(array $passengerCriteria, array $searchCriteria, array $selectedSegments, array $recommendation): array
    {
        try {
            $this->passengerCriteria = $passengerCriteria;
            $this->searchCriteria = $searchCriteria;
            $this->selectedSegments = $selectedSegments;
            $this->recommendation = $recommendation;

            $response = $this->createCurlRequest();

            self::storeInFile("xml", "2.1-air-sell-from-recommendation-response", $response);

            $responseArray = $this->xmlResponseBodyToArray($response);

            self::storeInFile(
                "json", "2.1-air-sell-from-recommendation-response", json_encode($responseArray)
            );

            return $responseArray;

        } catch (\Exception $exception) {
            $e = "Error in SelFromRecommendationAction: " . $exception->getMessage();
            Log::error($e);
            echo $e;

            return ["error" => $e];
        }
    }

    protected function action(): string
    {
        return "http://webservices.amadeus.com/ITAREQ_05_2_IA";
    }

    protected function generateXMLRequest(): string
    {
        $xmlRequest = $this->xmlHeader() . '
              <soap:Body>
                <Air_SellFromRecommendation>
                  <messageActionDetails>
                    <messageFunctionDetails>
                      <messageFunction>183</messageFunction>
                      <additionalMessageFunction>M1</additionalMessageFunction>
                    </messageFunctionDetails>
                  </messageActionDetails>
                    ' . $this->generateItineraries() . '
                </Air_SellFromRecommendation>
              </soap:Body>
            </soap:Envelope>';

        self::storeInFile("xml", "2.1-air-sell-from-recommendation-request", $xmlRequest);

        return $xmlRequest;
    }

    protected function generateItineraries(): string
    {
        $xml = "";
        foreach ($this->searchCriteria as $index => $legs) {
            $xml .= "
            <itineraryDetails>
                <originDestinationDetails>
                  <origin>" . $legs["From"]['value'] . "</origin>
                  <destination>" . $legs["To"]['value'] . "</destination>
                </originDestinationDetails>
                <message>
                  <messageFunctionDetails>
                    <messageFunction>183</messageFunction>
                  </messageFunctionDetails>
                </message>
                " . $this->generateSegmentTags($index) . "
            </itineraryDetails>
            ";
        }

        return $xml;
    }

    protected function generateSegmentTags(int $legIndex): string
    {
        $xmlNode = "";

        $quantity = $this->getPassengerQuantity();
        $flightDetails = $this->selectedSegments[$legIndex]['groupOfFlights']['flightDetails'];

        if (isset($flightDetails['flightInformation'])) {
            $xmlNode .= $this->getSegmentXmlNode($flightDetails['flightInformation'], $quantity, $xmlNode);
        } else {
            foreach ($flightDetails as $index => $segment) {
                $xmlNode .= $this->getSegmentXmlNode($segment['flightInformation'], $quantity, $xmlNode);
            }
        }

        return $xmlNode;
    }

    protected function getPassengerQuantity()
    {
        $adultsAndChildren = Collection::wrap($this->passengerCriteria)
            ->whereIn("passengerTypeCode", ["ADT", "CHD"])
            ->values();

        return $adultsAndChildren->count();
    }

    /**
     * @param $flightInformation
     * @param $quantity
     * @param string $xmlNode
     * @return string
     */
    protected function getSegmentXmlNode($flightInformation, $quantity, string $xmlNode): string
    {
        $dateOfDeparture = $flightInformation['productDateTime']['dateOfDeparture'];
        $bordPointLocationId = $flightInformation['location'][0]['locationId'];
        $offPointLocationId = $flightInformation['location'][1]['locationId'];
        $marketingCarrier = $flightInformation['companyId']['marketingCarrier'];
        $flightNumber = $flightInformation['flightOrtrainNumber'];
        $bookingClass = $flightInformation['bookingClass'];

        return "
                    <segmentInformation>
                      <travelProductInformation>
                        <flightDate>
                          <departureDate>$dateOfDeparture</departureDate>
                        </flightDate>
                        <boardPointDetails>
                          <trueLocationId>$bordPointLocationId</trueLocationId>
                        </boardPointDetails>
                        <offpointDetails>
                          <trueLocationId>$offPointLocationId</trueLocationId>
                        </offpointDetails>
                        <companyDetails>
                          <marketingCompany>$marketingCarrier</marketingCompany>
                        </companyDetails>
                        <flightIdentification>
                          <flightNumber>$flightNumber</flightNumber>
                          <bookingClass>$bookingClass</bookingClass>
                        </flightIdentification>
                      </travelProductInformation>
                      <relatedproductInformation>
                        <quantity>$quantity</quantity>
                        <statusCode>NN</statusCode>
                      </relatedproductInformation>
                    </segmentInformation>
            ";
    }
}
